@@ -7,11 +7,8 @@
 #ifndef _PINYINDICTMANAGER_BROWSERDIALOG_H_
 #define _PINYINDICTMANAGER_BROWSERDIALOG_H_
 
-#ifdef USE_WEBKIT
-#include "ui_browserdialog_webkit.h"
-#else
+#include "config.h"
 #include "ui_browserdialog.h"
-#endif
 #include <QByteArray>
 #include <QDialog>
 #include <QMessageBox>
