@@ -12,47 +12,76 @@
 #include "../../modules/cloudpinyin/cloudpinyin_public.h"
 #include "config.h"
 #include "customphrase.h"
+#include "notifications_public.h"
 #include "pinyincandidate.h"
+#include "pinyinhelper_public.h"
+#include "punctuation_public.h"
+#include "spell_public.h"
 #include "workerthread.h"
 #include <algorithm>
-#include <boost/iostreams/stream_buffer.hpp>
 #include <cassert>
 #include <cstdint>
 #include <ctime>
 #include <exception>
+#include <fcitx-config/iniparser.h>
 #include <fcitx-config/rawconfig.h>
 #include <fcitx-utils/capabilityflags.h>
+#include <fcitx-utils/charutils.h>
+#include <fcitx-utils/event.h>
 #include <fcitx-utils/eventloopinterface.h>
+#include <fcitx-utils/fdstreambuf.h>
+#include <fcitx-utils/i18n.h>
 #include <fcitx-utils/key.h>
 #include <fcitx-utils/keysym.h>
+#include <fcitx-utils/log.h>
 #include <fcitx-utils/macros.h>
 #include <fcitx-utils/misc.h>
+#include <fcitx-utils/standardpaths.h>
 #include <fcitx-utils/stringutils.h>
 #include <fcitx-utils/textformatflags.h>
+#include <fcitx-utils/unixfd.h>
+#include <fcitx-utils/utf8.h>
+#include <fcitx/action.h>
 #include <fcitx/addoninstance.h>
 #include <fcitx/candidatelist.h>
 #include <fcitx/event.h>
+#include <fcitx/inputcontext.h>
+#include <fcitx/inputcontextmanager.h>
+#include <fcitx/inputcontextproperty.h>
 #include <fcitx/inputmethodengine.h>
 #include <fcitx/inputmethodentry.h>
+#include <fcitx/inputpanel.h>
 #include <fcitx/instance.h>
 #include <fcitx/statusarea.h>
 #include <fcitx/text.h>
 #include <fcitx/userinterface.h>
-#include <fmt/core.h>
+#include <fcitx/userinterfacemanager.h>
+#include <fcntl.h>
+#include <filesystem>
 #include <fstream>
 #include <functional>
 #include <future>
 #include <ios>
 #include <istream>
 #include <iterator>
+#include <libime/core/historybigram.h>
 #include <libime/core/languagemodel.h>
 #include <libime/core/triedictionary.h>
+#include <libime/core/userlanguagemodel.h>
+#include <libime/pinyin/pinyincontext.h>
 #include <libime/pinyin/pinyincorrectionprofile.h>
+#include <libime/pinyin/pinyindecoder.h>
+#include <libime/pinyin/pinyindictionary.h>
+#include <libime/pinyin/pinyinencoder.h>
 #include <libime/pinyin/pinyinime.h>
+#include <libime/pinyin/pinyinprediction.h>
+#include <libime/pinyin/shuangpinprofile.h>
 #include <list>
 #include <memory>
 #include <optional>
 #include <ostream>
+#include <quickphrase_public.h>
+#include <ranges>
 #include <regex>
 #include <stdexcept>
 #include <string>
@@ -62,40 +91,10 @@
 #include <unordered_set>
 #include <utility>
 #include <vector>
+
 #ifdef FCITX_HAS_LUA
-#include "luaaddon_public.h"
+#include <luaaddon_public.h>
 #endif
-#include "notifications_public.h"
-#include "pinyinhelper_public.h"
-#include "punctuation_public.h"
-#include "spell_public.h"
-#include <boost/iostreams/device/file_descriptor.hpp>
-#include <boost/iostreams/stream.hpp>
-#include <fcitx-config/iniparser.h>
-#include <fcitx-utils/charutils.h>
-#include <fcitx-utils/event.h>
-#include <fcitx-utils/i18n.h>
-#include <fcitx-utils/log.h>
-#include <fcitx-utils/standardpath.h>
-#include <fcitx-utils/utf8.h>
-#include <fcitx/action.h>
-#include <fcitx/inputcontext.h>
-#include <fcitx/inputcontextmanager.h>
-#include <fcitx/inputcontextproperty.h>
-#include <fcitx/inputpanel.h>
-#include <fcitx/userinterfacemanager.h>
-#include <fcntl.h>
-#include <fmt/chrono.h>
-#include <fmt/format.h>
-#include <libime/core/historybigram.h>
-#include <libime/core/userlanguagemodel.h>
-#include <libime/pinyin/pinyincontext.h>
-#include <libime/pinyin/pinyindecoder.h>
-#include <libime/pinyin/pinyindictionary.h>
-#include <libime/pinyin/pinyinencoder.h>
-#include <libime/pinyin/pinyinprediction.h>
-#include <libime/pinyin/shuangpinprofile.h>
-#include <quickphrase_public.h>
 
 namespace fcitx {
 
@@ -211,16 +210,23 @@ void PinyinEngine::initPredict(InputContext *inputContext) {
     auto *state = inputContext->propertyFor(&factory_);
     auto &context = state->context_;
     auto lmState = context.state();
-    state->predictWords_ = context.selectedWords();
-    auto words =
-        prediction_.predict(lmState, context.selectedWords(),
-                            context.selectedWordsWithPinyin().back().second,
-                            *config_.predictionSize);
+    auto selected = context.selectedWords();
+    if (selected.empty()) {
+        return;
+    }
+
+    const auto selectPinyin = context.selectedWordsWithPinyin();
+    const std::string lastEncodedPinyin =
+        !selectPinyin.empty() ? selectPinyin.back().second : std::string();
+
+    auto words = prediction_.predict(lmState, selected, lastEncodedPinyin,
+                                     *config_.predictionSize);
     if (auto candidateList = predictCandidateList(this, words)) {
         auto &inputPanel = inputContext->inputPanel();
+        state->predictWords_ = std::move(selected);
         inputPanel.setCandidateList(std::move(candidateList));
     } else {
-        state->predictWords_.reset();
+        return;
     }
     inputContext->updatePreedit();
     inputContext->updateUserInterface(UserInterfaceComponent::InputPanel);
@@ -451,7 +457,7 @@ void PinyinEngine::updateUI(InputContext *inputContext) {
                     result.evaluate([this, inputContext](std::string_view key) {
                         return evaluateCustomPhrase(inputContext, key);
                     });
-                if (customCandidateSet.count(phrase)) {
+                if (customCandidateSet.contains(phrase)) {
                     continue;
                 }
                 customCandidateSet.insert(phrase);
@@ -484,8 +490,8 @@ void PinyinEngine::updateUI(InputContext *inputContext) {
                 *config_.cloudPinyinIndex - 1);
             if (!cand->filled() ||
                 (!cand->word().empty() &&
-                 !customCandidateSet.count(cand->word()) &&
-                 !context.candidatesToCursorSet().count(cand->word()))) {
+                 !customCandidateSet.contains(cand->word()) &&
+                 !context.candidatesToCursorSet().contains(cand->word()))) {
                 customCandidateSet.insert(cand->word());
                 extraCandidates.push_back(std::move(cand));
                 cloud = std::prev(extraCandidates.end());
@@ -526,7 +532,7 @@ void PinyinEngine::updateUI(InputContext *inputContext) {
 
                 int position = hasUpper ? 0 : 1;
                 for (const auto &result : results) {
-                    if (customCandidateSet.count(result)) {
+                    if (customCandidateSet.contains(result)) {
                         continue;
                     }
                     extraCandidates.push_back(
@@ -538,8 +544,8 @@ void PinyinEngine::updateUI(InputContext *inputContext) {
         /// }}}
 
         /// Create stroke candidate {{{
-        if (pinyinhelper() && context.selectedLength() == 0 &&
-            isStroke(context.userInput())) {
+        if (*config_.strokeCandidateEnabled && pinyinhelper() &&
+            context.selectedLength() == 0 && isStroke(context.userInput())) {
             int limit = (context.userInput().size() + 4) / 5;
             limit = std::min(limit, 3);
             auto results = pinyinhelper()->call<IPinyinHelper::lookupStroke>(
@@ -594,7 +600,7 @@ void PinyinEngine::updateUI(InputContext *inputContext) {
         for (size_t idx = 0, end = candidates.size(); idx < end; idx++) {
             const auto &candidate = candidates[idx];
             auto candidateString = candidate.toString();
-            if (customCandidateSet.count(candidateString)) {
+            if (customCandidateSet.contains(candidateString)) {
                 continue;
             }
             std::vector<std::string> luaExtraCandidates;
@@ -683,14 +689,11 @@ PinyinEngine::PinyinEngine(Instance *instance)
             libime::DefaultLanguageModelResolver::instance()
                 .languageModelFileForLanguage("zh_CN")));
 
-    const auto &standardPath = StandardPath::global();
+    const auto &standardPath = StandardPaths::global();
     auto systemDictFile =
-        standardPath.open(StandardPath::Type::Data, "libime/sc.dict", O_RDONLY);
+        standardPath.open(StandardPathsType::Data, "libime/sc.dict");
     if (systemDictFile.isValid()) {
-        boost::iostreams::stream_buffer<
-            boost::iostreams::file_descriptor_source>
-            buffer(systemDictFile.fd(),
-                   boost::iostreams::file_descriptor_flags::never_close_handle);
+        IFDStreamBuf buffer(systemDictFile.fd());
         std::istream in(&buffer);
         ime_->dict()->load(libime::PinyinDictionary::SystemDict, in,
                            libime::PinyinDictFormat::Binary);
@@ -703,18 +706,15 @@ PinyinEngine::PinyinEngine(Instance *instance)
     prediction_.setPinyinDictionary(ime_->dict());
 
     do {
-        auto file = standardPath.openUser(StandardPath::Type::PkgData,
-                                          "pinyin/user.dict", O_RDONLY);
+        auto file =
+            standardPath.open(StandardPathsType::PkgData, "pinyin/user.dict");
 
-        if (file.fd() < 0) {
+        if (!file.isValid()) {
             break;
         }
 
         try {
-            boost::iostreams::stream_buffer<
-                boost::iostreams::file_descriptor_source>
-                buffer(file.fd(), boost::iostreams::file_descriptor_flags::
-                                      never_close_handle);
+            IFDStreamBuf buffer(file.fd());
             std::istream in(&buffer);
             ime_->dict()->load(libime::PinyinDictionary::UserDict, in,
                                libime::PinyinDictFormat::Binary);
@@ -723,14 +723,12 @@ PinyinEngine::PinyinEngine(Instance *instance)
         }
     } while (0);
     do {
-        auto file = standardPath.openUser(StandardPath::Type::PkgData,
-                                          "pinyin/user.history", O_RDONLY);
+        auto file =
+            standardPath.open(StandardPathsType::PkgData, "pinyin/user.history",
+                              StandardPathsMode::User);
 
         try {
-            boost::iostreams::stream_buffer<
-                boost::iostreams::file_descriptor_source>
-                buffer(file.fd(), boost::iostreams::file_descriptor_flags::
-                                      never_close_handle);
+            IFDStreamBuf buffer(file.fd());
             std::istream in(&buffer);
             ime_->model()->load(in);
         } catch (const std::exception &e) {
@@ -821,20 +819,17 @@ PinyinEngine::PinyinEngine(Instance *instance)
 
 PinyinEngine::~PinyinEngine() {}
 
-void PinyinEngine::loadSymbols(const StandardPathFile &file) {
-    if (file.fd() < 0) {
+void PinyinEngine::loadSymbols(const UnixFD &file) {
+    if (!file.isValid()) {
         return;
     }
-    boost::iostreams::stream_buffer<boost::iostreams::file_descriptor_source>
-        buffer(file.fd(),
-               boost::iostreams::file_descriptor_flags::never_close_handle);
+    IFDStreamBuf buffer(file.fd());
     std::istream in(&buffer);
     try {
-        PINYIN_DEBUG() << "Loading symbol dict " << file.path();
+        PINYIN_DEBUG() << "Loading symbol dict.";
         symbols_.load(in);
     } catch (const std::exception &e) {
-        PINYIN_ERROR() << "Failed to load symbol dict " << file.path() << ": "
-                       << e.what();
+        PINYIN_ERROR() << "Failed to load symbol dict: " << e.what();
     }
 }
 
@@ -867,23 +862,23 @@ void PinyinEngine::loadDict(const std::string &fullPath,
 }
 
 void PinyinEngine::loadBuiltInDict() {
-    const auto &standardPath = StandardPath::global();
+    const auto &standardPath = StandardPaths::global();
     {
-        auto file = standardPath.open(StandardPath::Type::PkgData,
-                                      "pinyin/symbols", O_RDONLY);
+        auto file =
+            standardPath.open(StandardPathsType::PkgData, "pinyin/symbols");
         loadSymbols(file);
     }
     {
-        auto file = standardPath.locate(StandardPath::Type::PkgData,
+        auto file = standardPath.locate(StandardPathsType::PkgData,
                                         "pinyin/chaizi.dict");
         loadDict(file, persistentTask_);
     }
     {
         auto file =
-            standardPath.locate(StandardPath::Type::Data, "libime/extb.dict");
+            standardPath.locate(StandardPathsType::Data, "libime/extb.dict");
         // Try again with absolute libime path.
         if (file.empty()) {
-            file = standardPath.locate(StandardPath::Type::Data,
+            file = standardPath.locate(StandardPathsType::Data,
                                        LIBIME_INSTALL_PKGDATADIR "/extb.dict");
         }
         loadDict(file, persistentTask_);
@@ -895,13 +890,18 @@ void PinyinEngine::loadBuiltInDict() {
 }
 
 void PinyinEngine::loadExtraDict() {
-    const auto &standardPath = StandardPath::global();
+    const auto &standardPath = StandardPaths::global();
     auto files =
-        standardPath.locate(StandardPath::Type::PkgData, "pinyin/dictionaries",
-                            filter::Suffix(".dict"));
-    auto disableFiles =
-        standardPath.locate(StandardPath::Type::PkgData, "pinyin/dictionaries",
-                            filter::Suffix(".dict.disable"));
+        standardPath.locate(StandardPathsType::PkgData, "pinyin/dictionaries",
+                            pathfilter::extension(".dict"));
+    std::unordered_set<std::filesystem::path> disableFilesSet;
+    for (const auto &item :
+         standardPath.locate(StandardPathsType::PkgData, "pinyin/dictionaries",
+                             pathfilter::extension(".disable")) |
+             std::views::transform(
+                 [](const auto &item) { return item.first.stem(); })) {
+        disableFilesSet.insert(item);
+    }
     FCITX_ASSERT(ime_->dict()->dictSize() >=
                  libime::TrieDictionary::UserDict + NumBuiltInDict + 1)
         << "Dict size: " << ime_->dict()->dictSize();
@@ -909,7 +909,7 @@ void PinyinEngine::loadExtraDict() {
     ime_->dict()->removeFrom(libime::TrieDictionary::UserDict + NumBuiltInDict +
                              1);
     for (auto &file : files) {
-        if (disableFiles.count(stringutils::concat(file.first, ".disable"))) {
+        if (disableFilesSet.contains(file.first)) {
             PINYIN_DEBUG() << "Dictionary: " << file.first << " is disabled.";
             continue;
         }
@@ -919,19 +919,16 @@ void PinyinEngine::loadExtraDict() {
 }
 
 void PinyinEngine::loadCustomPhrase() {
-    const auto &standardPath = StandardPath::global();
-    auto file = standardPath.open(StandardPath::Type::PkgData,
-                                  "pinyin/customphrase", O_RDONLY);
+    const auto &standardPath = StandardPaths::global();
+    auto file =
+        standardPath.open(StandardPathsType::PkgData, "pinyin/customphrase");
     if (!file.isValid()) {
         customPhrase_.clear();
         return;
     }
 
     try {
-        boost::iostreams::stream_buffer<
-            boost::iostreams::file_descriptor_source>
-            buffer(file.fd(),
-                   boost::iostreams::file_descriptor_flags::never_close_handle);
+        IFDStreamBuf buffer(file.fd());
         std::istream in(&buffer);
         customPhrase_.load(in, true);
     } catch (const std::exception &e) {
@@ -956,10 +953,10 @@ void PinyinEngine::populateConfig() {
                             "prediction? You can always toggle it later in "
                             "configuration.");
                 } else {
-                    msg = fmt::format(
-                        _("Do you want to enable cloudpinyin now for better "
-                          "prediction? You can always toggle it later in "
-                          "configuration or by pressing {}."),
+                    msg = _(
+                        "Do you want to enable cloudpinyin now for better "
+                        "prediction? You can always toggle it later in "
+                        "configuration or by pressing {}.",
                         Key::keyListToString(key, KeyStringFormat::Localized));
                 }
                 std::vector<std::string> actions = {"yes", _("Yes"), "no",
@@ -981,6 +978,7 @@ void PinyinEngine::populateConfig() {
         });
     }
     ime_->setNBest(*config_.nbest);
+    ime_->setWordCandidateLimit(*config_.wordCandidateLimit);
     ime_->setPartialLongWordLimit(*config_.longWordLimit);
     ime_->setPreeditMode(*config_.showActualPinyinInPreedit
                              ? libime::PinyinPreeditMode::Pinyin
@@ -1010,6 +1008,7 @@ void PinyinEngine::populateConfig() {
     SET_FUZZY_FLAG(c, C_CH)
     SET_FUZZY_FLAG(f, F_H)
     SET_FUZZY_FLAG(l, L_N)
+    SET_FUZZY_FLAG(r, L_R)
     SET_FUZZY_FLAG(s, S_SH)
     SET_FUZZY_FLAG(z, Z_ZH)
 
@@ -1033,14 +1032,11 @@ void PinyinEngine::populateConfig() {
     ime_->setFuzzyFlags(flags);
 
     if (*config_.shuangpinProfile == ShuangpinProfileEnum::Custom) {
-        auto file = StandardPath::global().open(StandardPath::Type::PkgConfig,
-                                                "pinyin/sp.dat", O_RDONLY);
+        auto file = StandardPaths::global().open(StandardPathsType::PkgConfig,
+                                                 "pinyin/sp.dat");
         if (file.isValid()) {
             try {
-                boost::iostreams::stream_buffer<
-                    boost::iostreams::file_descriptor_source>
-                    buffer(file.fd(), boost::iostreams::file_descriptor_flags::
-                                          never_close_handle);
+                IFDStreamBuf buffer(file.fd());
                 std::istream in(&buffer);
                 ime_->setShuangpinProfile(
                     std::make_shared<libime::ShuangpinProfile>(
@@ -1570,12 +1566,9 @@ void PinyinEngine::forgetCandidate(InputContext *inputContext, size_t index) {
 
 void PinyinEngine::saveCustomPhrase() {
     instance_->eventDispatcher().scheduleWithContext(watch(), [this]() {
-        StandardPath::global().safeSave(
-            StandardPath::Type::PkgData, "pinyin/customphrase", [this](int fd) {
-                boost::iostreams::stream_buffer<
-                    boost::iostreams::file_descriptor_sink>
-                    buffer(fd, boost::iostreams::file_descriptor_flags::
-                                   never_close_handle);
+        StandardPaths::global().safeSave(
+            StandardPathsType::PkgData, "pinyin/customphrase", [this](int fd) {
+                OFDStreamBuf buffer(fd);
                 std::ostream out(&buffer);
                 try {
                     customPhrase_.save(out);
@@ -1814,10 +1807,10 @@ bool PinyinEngine::handlePunc(KeyEvent &event,
         std::string text;
         if (!output.empty()) {
             if (!altOutput.empty()) {
-                text = fmt::format(_("Press {} for {} and Return for {}"),
-                                   keyString, output, altOutput);
+                text = _("Press {} for {} and Return for {}", keyString, output,
+                         altOutput);
             } else {
-                text = fmt::format(_("Press {} for {}"), keyString, altOutput);
+                text = _("Press {} for {}", keyString, output);
             }
         }
         quickphrase()->call<IQuickPhrase::trigger>(
@@ -1963,7 +1956,13 @@ void PinyinEngine::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
     if (state->predictWords_) {
         resetPredict(inputContext);
         if (event.key().check(FcitxKey_Escape) ||
-            (isAndroid() && event.key().check(FcitxKey_BackSpace)) ||
+            (event.key().check(FcitxKey_BackSpace) &&
+             (*config_.backspaceBehaviorOnPrediction ==
+                  BackspaceBehaviorOnPrediction::OnlyClearCandidates ||
+              (*config_.backspaceBehaviorOnPrediction ==
+                   BackspaceBehaviorOnPrediction::
+                       BackspaceWhenNotUsingOnScreenKeyboard &&
+               event.isVirtual()))) ||
             event.key().check(FcitxKey_Delete)) {
             event.filterAndAccept();
             return;
@@ -1980,9 +1979,9 @@ void PinyinEngine::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
         // char.
         char chr = keyChr.get();
         return (!state->context_.empty() &&
-                shuangpinProfile->validInput().count(chr)) ||
+                shuangpinProfile->validInput().contains(chr)) ||
                (state->context_.empty() &&
-                shuangpinProfile->validInitial().count(chr));
+                shuangpinProfile->validInitial().contains(chr));
     };
 
     if (!event.key().hasModifier() && quickphrase() &&
@@ -2019,14 +2018,20 @@ void PinyinEngine::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
         (event.key().check(FcitxKey_apostrophe) && !state->context_.empty()) ||
         checkSp(event, state)) {
         // first v, use it to trigger quickphrase
-        if (!state->context_.useShuangpin() && quickphrase() &&
-            *config_.useVAsQuickphrase && event.key().check(FcitxKey_v) &&
+        if (*config_.useVAsQuickphrase && quickphrase() &&
             state->context_.empty()) {
+            const bool isSp = state->context_.useShuangpin();
+            if ((event.key().check(FcitxKey_v) && !isSp) ||
+                (event.key().check(FcitxKey_V) && isSp)) {
+                const std::string leadingString =
+                    Key::keySymToUTF8(event.key().sym());
 
-            quickphrase()->call<IQuickPhrase::trigger>(
-                inputContext, "", "v", "", "", Key(FcitxKey_None));
-            event.filterAndAccept();
-            return;
+                quickphrase()->call<IQuickPhrase::trigger>(
+                    inputContext, "", leadingString, "", "",
+                    Key(FcitxKey_None));
+                event.filterAndAccept();
+                return;
+            }
         }
         event.filterAndAccept();
         if (!state->context_.type(keyStr.get())) {
@@ -2222,13 +2227,10 @@ void PinyinEngine::doReset(InputContext *inputContext) const {
 
 void PinyinEngine::save() {
     safeSaveAsIni(config_, "conf/pinyin.conf");
-    const auto &standardPath = StandardPath::global();
+    const auto &standardPath = StandardPaths::global();
     standardPath.safeSave(
-        StandardPath::Type::PkgData, "pinyin/user.dict", [this](int fd) {
-            boost::iostreams::stream_buffer<
-                boost::iostreams::file_descriptor_sink>
-                buffer(fd, boost::iostreams::file_descriptor_flags::
-                               never_close_handle);
+        StandardPathsType::PkgData, "pinyin/user.dict", [this](int fd) {
+            OFDStreamBuf buffer(fd);
             std::ostream out(&buffer);
             try {
                 ime_->dict()->save(libime::PinyinDictionary::UserDict, out,
@@ -2240,11 +2242,8 @@ void PinyinEngine::save() {
             }
         });
     standardPath.safeSave(
-        StandardPath::Type::PkgData, "pinyin/user.history", [this](int fd) {
-            boost::iostreams::stream_buffer<
-                boost::iostreams::file_descriptor_sink>
-                buffer(fd, boost::iostreams::file_descriptor_flags::
-                               never_close_handle);
+        StandardPathsType::PkgData, "pinyin/user.history", [this](int fd) {
+            OFDStreamBuf buffer(fd);
             std::ostream out(&buffer);
             try {
                 ime_->model()->save(out);
